import path from 'path';
import mime from 'mime-types';
import { getFileExtension, } from 'itk-wasm';
import mimeToMeshIo from './mime-to-mesh-io.js';
import extensionToMeshIo from './extension-to-mesh-io.js';
import meshIoIndexNode from './mesh-io-index-node.js';
/**
 * Read a mesh file format and convert it to the itk-wasm file format
 *
 * @param {string} serializedMesh - Path to input mesh serialized in the file format
 * @param {ReadMeshOptions} options - options to cast resulting mesh type or to only read mesh metadata
 *
 * @returns {Promise<Mesh>} - Mesh result
 */
async function readMeshNode(serializedMesh, options = {}) {
    const absoluteFilePath = path.resolve(serializedMesh);
    const mimeType = mime.lookup(absoluteFilePath);
    const extension = getFileExtension(absoluteFilePath);
    let io = null;
    if (mimeType && mimeToMeshIo.has(mimeType)) {
        io = mimeToMeshIo.get(mimeType);
    }
    else if (extensionToMeshIo.has(extension)) {
        io = extensionToMeshIo.get(extension);
    }
    else {
        for (const readerWriter of meshIoIndexNode.values()) {
            if (readerWriter[0] !== null) {
                let { couldRead, mesh } = await readerWriter[0](absoluteFilePath, { informationOnly: options.informationOnly });
                if (couldRead) {
                    return mesh;
                }
            }
        }
    }
    if (io === null) {
        throw Error('Could not find IO for: ' + absoluteFilePath);
    }
    const readerWriter = meshIoIndexNode.get(io);
    const reader = readerWriter[0];
    let { couldRead, mesh } = await reader(absoluteFilePath, { informationOnly: options.informationOnly });
    if (!couldRead) {
        throw Error('Could not read: ' + absoluteFilePath);
    }
    return mesh;
}
export default readMeshNode;
//# sourceMappingURL=read-mesh-node.js.map