import { getFileExtension, } from "itk-wasm";
import mimeToPointSetIo from "./mime-to-point-set-io.js";
import extensionToPointSetIo from "./extension-to-point-set-io.js";
import pointSetIoIndex from "./point-set-io-index.js";
/**
 * Read a point set file format and convert it to the itk-wasm file format
 *
 * @param {webWorker} null | webWorker - Web worker to run the pipeline or null to run it in a new worker
 * @param {File | BinaryFile} serializedPointSet - Input point set serialized in the file format
 * @param {ReadPointSetOptions} options - options to cast the resulting point set type or to only read point set metadata
 *
 * @returns {Promise<ReadPointSetResult>} - result object with the point set and the web worker used
 */
async function readPointSet(serializedPointSet, options = {}) {
    const mimeType = serializedPointSet.type ?? "";
    const fileName = serializedPointSet.name ??
        serializedPointSet.path ??
        "fileName";
    const extension = getFileExtension(fileName).toLowerCase();
    let usedWebWorker = options?.webWorker;
    let serializedPointSetFile = serializedPointSet;
    if (serializedPointSet instanceof Blob) {
        const serializedPointSetBuffer = await serializedPointSet.arrayBuffer();
        serializedPointSetFile = {
            path: serializedPointSet.name,
            data: new Uint8Array(serializedPointSetBuffer),
        };
    }
    let io = null;
    if (mimeType && mimeToPointSetIo.has(mimeType)) {
        io = mimeToPointSetIo.get(mimeType);
    }
    else if (extensionToPointSetIo.has(extension)) {
        io = extensionToPointSetIo.get(extension);
    }
    else {
        for (const readerWriter of pointSetIoIndex.values()) {
            if (readerWriter[0] !== null) {
                let { webWorker: testWebWorker, couldRead, pointSet, } = await readerWriter[0]({
                    path: serializedPointSetFile.path,
                    data: serializedPointSetFile.data.slice(),
                }, {
                    informationOnly: options.informationOnly,
                    webWorker: usedWebWorker,
                    noCopy: options?.noCopy,
                });
                usedWebWorker = testWebWorker;
                if (couldRead) {
                    return { webWorker: usedWebWorker, pointSet };
                }
            }
        }
    }
    if (!io) {
        throw Error("Could not find IO for: " + fileName);
    }
    const readerWriter = pointSetIoIndex.get(io);
    const reader = readerWriter[0];
    let { webWorker: testWebWorker, couldRead, pointSet, } = await reader(serializedPointSetFile, {
        informationOnly: options.informationOnly,
        webWorker: usedWebWorker,
        noCopy: options?.noCopy,
    });
    usedWebWorker = testWebWorker;
    if (!couldRead) {
        throw Error("Could not read: " + fileName);
    }
    return { webWorker: usedWebWorker, pointSet };
}
export default readPointSet;
//# sourceMappingURL=read-point-set.js.map