import path from "path";
import { getFileExtension } from "itk-wasm";
import mimeToPointSetIo from "./mime-to-point-set-io.js";
import extensionToPointSetIo from "./extension-to-point-set-io.js";
import pointSetIoIndexNode from "./point-set-io-index-node.js";
/**
 * Write a point set to a serialized file format and from an the itk-wasm PointSet
 *
 * @param {PointSet} point set - Input point set
 * @param {string} serializedPointSet - Output point set serialized in the file format.
 * @param {WritePointSetOptions} options - options object
 *
 * @returns {void} - result object
 */
async function writePointSetNode(pointSet, serializedPointSet, options = {}) {
    const absoluteFilePath = path.resolve(serializedPointSet);
    const mimeType = options.mimeType;
    const extension = getFileExtension(absoluteFilePath);
    let inputPointSet = pointSet;
    let io = null;
    if (typeof mimeType !== "undefined" && mimeToPointSetIo.has(mimeType)) {
        io = mimeToPointSetIo.get(mimeType);
    }
    else if (extensionToPointSetIo.has(extension)) {
        io = extensionToPointSetIo.get(extension);
    }
    else {
        for (const readerWriter of pointSetIoIndexNode.values()) {
            if (readerWriter[1] !== null) {
                let { couldWrite } = await readerWriter[1](inputPointSet, absoluteFilePath, {
                    useCompression: options.useCompression,
                    binaryFileType: options.binaryFileType,
                });
                if (couldWrite) {
                    return;
                }
            }
        }
    }
    if (io === null) {
        throw Error("Could not find IO for: " + absoluteFilePath);
    }
    const readerWriter = pointSetIoIndexNode.get(io);
    const writer = readerWriter[1];
    let { couldWrite } = await writer(inputPointSet, absoluteFilePath, {
        useCompression: options.useCompression,
    });
    if (!couldWrite) {
        throw Error("Could not write: " + absoluteFilePath);
    }
}
export default writePointSetNode;
//# sourceMappingURL=write-point-set-node.js.map