import { getFileExtension } from "itk-wasm";
import mimeToPointSetIo from "./mime-to-point-set-io.js";
import extensionToPointSetIo from "./extension-to-point-set-io.js";
import pointSetIoIndex from "./point-set-io-index.js";
/**
 * Write an itk-wasm PointSet converted to an serialized point set file format
 *
 * @param {PointSet} pointSet - Input point set
 * @param {string} serializedPointSet - Output point set serialized in the file format.
 * @param {WritePointSetOptions} options - options object
 *
 * @returns {Promise<WritePointSetResult>} - result object
 */
async function writePointSet(pointSet, serializedPointSet, options = {}) {
    let inputPointSet = pointSet;
    const mimeType = options.mimeType;
    const extension = getFileExtension(serializedPointSet).toLowerCase();
    let usedWebWorker = options.webWorker;
    let io = null;
    if (typeof mimeType !== "undefined" && mimeToPointSetIo.has(mimeType)) {
        io = mimeToPointSetIo.get(mimeType);
    }
    else if (extensionToPointSetIo.has(extension)) {
        io = extensionToPointSetIo.get(extension);
    }
    else {
        for (const readerWriter of pointSetIoIndex.values()) {
            if (readerWriter[1] !== null) {
                let { webWorker: testWebWorker, couldWrite, serializedPointSet: serializedPointSetBuffer, } = await readerWriter[1](inputPointSet, serializedPointSet, options);
                usedWebWorker = testWebWorker;
                if (couldWrite) {
                    return {
                        webWorker: usedWebWorker,
                        serializedPointSet: serializedPointSetBuffer,
                    };
                }
            }
        }
    }
    if (!io) {
        throw Error("Could not find IO for: " + serializedPointSet);
    }
    const readerWriter = pointSetIoIndex.get(io);
    const writer = readerWriter[1];
    let { webWorker: testWebWorker, couldWrite, serializedPointSet: serializedPointSetBuffer, } = await writer(inputPointSet, serializedPointSet, options);
    usedWebWorker = testWebWorker;
    if (!couldWrite) {
        throw Error("Could not write: " + serializedPointSet);
    }
    const result = {
        webWorker: usedWebWorker,
        serializedPointSet: serializedPointSetBuffer,
    };
    return result;
}
export default writePointSet;
//# sourceMappingURL=write-point-set.js.map